@@ -296,6 +296,37 @@ impl<F> Io<F> {
     pub fn set_disconnect_timeout(&self, timeout: Millis) {
         self.0 .0.disconnect_timeout.set(timeout);
     }
+
+    #[inline]
+    /// Clone current io object.
+    ///
+    /// Current io object becomes closed.
+    pub fn take(&mut self) -> Self {
+        let inner = Rc::new(IoState {
+            pool: self.0 .0.pool.clone(),
+            flags: Cell::new(
+                Flags::DSP_STOP
+                    | Flags::IO_STOPPED
+                    | Flags::IO_STOPPING
+                    | Flags::IO_STOPPING_FILTERS,
+            ),
+            error: Cell::new(None),
+            disconnect_timeout: Cell::new(Millis::ONE_SEC),
+            dispatch_task: LocalWaker::new(),
+            read_task: LocalWaker::new(),
+            write_task: LocalWaker::new(),
+            read_buf: Cell::new(None),
+            write_buf: Cell::new(None),
+            filter: Cell::new(NullFilter::get()),
+            handle: Cell::new(None),
+            on_disconnect: RefCell::new(Vec::new()),
+            keepalive: Cell::new(None),
+        });
+
+        let state = mem::replace(&mut self.0, IoRef(inner));
+        let filter = mem::replace(&mut self.1, FilterItem::Ptr(ptr::null_mut()));
+        Self(state, filter)
+    }
 }
 
 impl<F> Io<F> {
@@ -668,6 +699,39 @@ impl<F> AsRef<IoRef> for Io<F> {
     }
 }
 
+impl<F> Eq for Io<F> {}
+
+impl<F> PartialEq for Io<F> {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        self.0.eq(&other.0)
+    }
+}
+
+impl<F> hash::Hash for Io<F> {
+    #[inline]
+    fn hash<H: hash::Hasher>(&self, state: &mut H) {
+        self.0.hash(state);
+    }
+}
+
+impl<F> fmt::Debug for Io<F> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Io")
+            .field("open", &!self.is_closed())
+            .finish()
+    }
+}
+
+impl<F> Deref for Io<F> {
+    type Target = IoRef;
+
+    #[inline]
+    fn deref(&self) -> &Self::Target {
+        &self.0
+    }
+}
+
 impl<F> Drop for Io<F> {
     fn drop(&mut self) {
         self.remove_keepalive_timer();
@@ -696,23 +760,6 @@ impl<F> Drop for Io<F> {
     }
 }
 
-impl fmt::Debug for Io {
-    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        f.debug_struct("Io")
-            .field("open", &!self.is_closed())
-            .finish()
-    }
-}
-
-impl<F> Deref for Io<F> {
-    type Target = IoRef;
-
-    #[inline]
-    fn deref(&self) -> &Self::Target {
-        &self.0
-    }
-}
-
 /// OnDisconnect future resolves when socket get disconnected
 #[must_use = "OnDisconnect do nothing unless polled"]
 pub struct OnDisconnect {
