@@ -1,4 +1,4 @@
-use std::{any, fmt, io};
+use std::{any, fmt, hash, io};
 
 use ntex_bytes::{BufMut, BytesMut, PoolRef};
 use ntex_codec::{Decoder, Encoder};
@@ -190,6 +190,22 @@ impl IoRef {
     }
 }
 
+impl Eq for IoRef {}
+
+impl PartialEq for IoRef {
+    #[inline]
+    fn eq(&self, other: &Self) -> bool {
+        self.0.eq(&other.0)
+    }
+}
+
+impl hash::Hash for IoRef {
+    #[inline]
+    fn hash<H: hash::Hasher>(&self, state: &mut H) {
+        self.0.hash(state);
+    }
+}
+
 impl fmt::Debug for IoRef {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
         f.debug_struct("IoRef")
@@ -355,16 +371,14 @@ mod tests {
             self.inner.get_read_buf()
         }
 
-        fn release_read_buf(
-            &self,
-            io: &IoRef,
-            buf: BytesMut,
-            dst: &mut Option<BytesMut>,
-            new_bytes: usize,
-        ) -> io::Result<usize> {
-            let result = self.inner.release_read_buf(io, buf, dst, new_bytes)?;
+        fn release_read_buf(&self, buf: BytesMut) {
+            self.inner.release_read_buf(buf)
+        }
+
+        fn process_read_buf(&self, io: &IoRef, n: usize) -> io::Result<(usize, usize)> {
+            let result = self.inner.process_read_buf(io, n)?;
             self.read_order.borrow_mut().push(self.idx);
-            self.in_bytes.set(self.in_bytes.get() + result);
+            self.in_bytes.set(self.in_bytes.get() + result.1);
             Ok(result)
         }
 
